@@ -1,5 +1,5 @@
 /* font-ximage.c --- renders text to an XImage for use with OpenGL.
- * xscreensaver, Copyright (c) 2001 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 2001, 2003 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -17,13 +17,19 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <GL/gl.h>	/* only for GLfloat */
+
+#ifdef HAVE_COCOA
+# include "jwxyz.h"
+# include <OpenGL/gl.h>
+#else  /* !HAVE_COCOA */
+# include <X11/Xlib.h>
+# include <X11/Xutil.h>
+# include <GL/gl.h>	/* only for GLfloat */
+#endif /* !HAVE_COCOA */
 
 extern char *progname;
 
-#include <X11/Xutil.h>
+#include "font-ximage.h"
 
 #undef MAX
 #define MAX(a,b) ((a)>(b)?(a):(b))
@@ -31,6 +37,7 @@ extern char *progname;
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
+#if 0
 static Bool
 bigendian (void)
 {
@@ -38,13 +45,15 @@ bigendian (void)
   u.i = 1;
   return !u.c[0];
 }
+#endif
 
 /* return the next larger power of 2. */
 static int
 to_pow2 (int i)
 {
-  static unsigned int pow2[] = { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024,
-                                 2048, 4096, 8192, 16384, 32768, 65536 };
+  static const unsigned int pow2[] = { 
+    1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024,
+    2048, 4096, 8192, 16384, 32768, 65536 };
   int j;
   for (j = 0; j < countof(pow2); j++)
     if (pow2[j] >= i) return pow2[j];
@@ -199,9 +208,15 @@ text_to_ximage (Screen *screen, Visual *visual,
          things as necessary) OpenGL pretends everything is client-side, so
          we need to pack things in the right order for the client machine.
        */
+#if 0
+    /* #### Cherub says that the little-endian case must be taken on MacOSX,
+            or else the colors/alpha are the wrong way around.  How can
+            that be the case?
+     */
       if (bigendian())
         rpos = 24, gpos = 16, bpos =  8, apos =  0;
       else
+#endif
         rpos =  0, gpos =  8, bpos = 16, apos = 24;
 
       fg = (((unsigned long) (texture_fg[0] * 255.0) << rpos) |
@@ -222,8 +237,6 @@ text_to_ximage (Screen *screen, Visual *visual,
                      XGetPixel (ximage1, x, y2) ? fg : bg);
       }
 
-    free (ximage1->data);
-    ximage1->data = 0;
     XDestroyImage (ximage1);
 
 #if 0
